@@ -72,7 +72,8 @@
  * Globals
  */
 
-#define	RNGD_STAT_SLEEP_TIME 3600
+#define STATS_INTERVAL_DFL 3600
+#define STATS_INTERVAL_MAX 604800
 
 /* Statistics */
 struct rng_stats rng_stats;
@@ -118,7 +119,12 @@ static struct argp_option options[] = {
 	  "(default: " PIDFILE ")" },
 
 	{ "rng-buffers", 'B', "n", 0,
-	  "Number of buffers (default: 3),  0 < n <= " XSTR(MAX_RNG_BUFFERS) },
+	  "Number of buffers (default: 3), 0 < n <= " XSTR(MAX_RNG_BUFFERS) },
+
+	{ "stats-interval", 'S', "n", 0,
+	  "Interval between logging statistics to syslog (default: "
+	  XSTR(STATS_INTERVAL_DFL) "), 0 <= n <= "
+	  XSTR(STATS_INTERVAL_MAX) ", 0 to disable" },
 
 	/**/ { 0, 0, 0, 0, "Input (entropy source) control" }, /* ========= */
 
@@ -191,6 +197,7 @@ static struct arguments default_arguments = {
 	.rng_name	= DEVHWRANDOM,
 	.random_name	= DEVRANDOM,
 	.pidfile_name	= PIDFILE,
+	.stats_interval	= STATS_INTERVAL_DFL,
 	.feed_interval	= 60,
 	.random_step	= 64,
 	.fill_watermark = -50,
@@ -291,6 +298,18 @@ static error_t parse_opt (int key, char *arg, struct argp_state *state)
 		arguments->pidfile_name = arg;
 		break;
 
+	case 'S': {
+		long n;
+		char *ep;
+
+		n = strtol(arg, &ep, 10);
+		if (!*arg || *ep || n < 0 || n > STATS_INTERVAL_MAX)
+			argp_usage(state);
+		else
+			arguments->stats_interval = n;
+		break;
+	}
+
 	case ARGP_RNGD_CMDLINE_TIMEOUT: 	/* --timeout */
 		message (LOG_WARNING, "Warning: --timeout is deprecated, "
 			 "use --feed-interval instead");
@@ -654,6 +673,7 @@ int main(int argc, char **argv)
 {
 	int fd;
 	pthread_t t1,t2,t3;
+	int sleepinterval;
 	int sleeptime;
 
 	kernel = kernel_mode();
@@ -716,12 +736,15 @@ int main(int argc, char **argv)
 	 * All we can do now is spin around waiting for a hit to the head.
 	 * Dump stats every hour, and at exit...
 	 */
-	sleeptime = RNGD_STAT_SLEEP_TIME;
+	sleepinterval = arguments->stats_interval ?
+	    arguments->stats_interval : STATS_INTERVAL_MAX;
+	sleeptime = sleepinterval;
 	while (!gotsigterm) {
 		sleeptime = sleep(sleeptime);
-		if ((sleeptime == 0) || gotsigusr1 || gotsigterm) {
+		if ((arguments->stats_interval && sleeptime == 0) ||
+		    gotsigusr1 || gotsigterm) {
 			dump_rng_stats();
-			sleeptime = RNGD_STAT_SLEEP_TIME;
+			sleeptime = sleepinterval;
 			gotsigusr1 = 0;
 		}
 	}
